/*
 * Copyright (C) 2023 Uniontech Software Technology Co., Ltd.
 *
 * Author:     gongheng<gongheng@uniontech.com>
 *
 * Maintainer: gongheng<gongheng@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TPMOPERATOR_H
#define TPMOPERATOR_H

#include <QObject>
#include <QThread>

class TPMOperator : public QObject
{
    Q_OBJECT
public:
    ~TPMOperator();
    static TPMOperator *instance();
    bool checkTPMAvailable();
    bool getRandom(int size, QString &output);
    bool isSupportAlgo(const QString &algoName);
    void asynEncrypt(const QString hashAlgo, const QString keyAlgo, const QString keyPin, const QString password);
    void asynDecrypt(const QString &keyPin);

signals:
    void sigStartAsynEncrypt(const QString hashAlgo, const QString keyAlgo, const QString keyPin, const QString password);
    void sigAsynEncryptFinished(bool result);
    void sigStartAsynDecrypt(const QString &keyPin);
    void sigAsynDecryptFinished(bool result, const QString &password);

private:
    explicit TPMOperator(QObject *parent = nullptr);

private:
    QThread workerThread;
};

#endif // TPMOPERATOR_H
